@@ -1,7 +1,7 @@
-#include <sys/types.h>
-#include <pwd.h>
 #include <errno.h>
+#include <pwd.h>
 #include <stdio.h>
+#include <sys/types.h>
 
 int main(int argc, char **argv) {
   setpwent();
@@ -9,8 +9,7 @@ int main(int argc, char **argv) {
     errno = 0;
     struct passwd *pw = getpwent();
     if (!pw) {
-      if (errno != 0)
-        printf("ERRNO: %u %m\n", errno);
+      if (errno != 0) printf("ERRNO: %u %m\n", errno);
       break;
     }
   }
